!! ##############################################################################################
!!
!! Copyright 2012 CNRS, INPT
!!  
!! This file is part of qr_mumps.
!!  
!! qr_mumps is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! qr_mumps is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the qr_mumps/doc directory.
!!
!! ##############################################################################################


!! ##############################################################################################
!> @file qrm_methods_mod.F90
!! This file contains a module with all the generic interfaces for the typed methods routines.
!!
!! $Date: 2016-01-29 22:22:30 +0100 (Fri, 29 Jan 2016) $
!! $Author: abuttari $
!! $Version: 1.1$
!! $Revision: 2075 $
!!
!! ##############################################################################################


!> @brief This module contains generic methods
module dqrm_methods_mod
  

  interface dqrm_least_squares
     module procedure dqrm_least_squares2dw, dqrm_least_squares1dw
  end interface dqrm_least_squares

  interface qrm_least_squares
     subroutine dqrm_least_squares2d(qrm_mat, b, x)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0)) :: b(:,:), x(:,:)
     end subroutine dqrm_least_squares2d
     subroutine dqrm_least_squares1d(qrm_mat, b, x)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0)) :: b(:), x(:)
     end subroutine dqrm_least_squares1d
  end interface qrm_least_squares
  


  interface dqrm_min_norm
     module procedure dqrm_min_norm2dw, dqrm_min_norm1dw
  end interface dqrm_min_norm

  interface qrm_min_norm
     subroutine dqrm_min_norm2d(qrm_mat, b, x)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0)) :: b(:,:), x(:,:)
     end subroutine dqrm_min_norm2d
     subroutine dqrm_min_norm1d(qrm_mat, b, x)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0)) :: b(:), x(:)
     end subroutine dqrm_min_norm1d
  end interface qrm_min_norm
  

  interface dqrm_residual_norm
     module procedure dqrm_residual_norm2dw, dqrm_residual_norm1dw
  end interface dqrm_residual_norm

  interface qrm_residual_norm
     subroutine dqrm_residual_norm2d(qrm_mat, b, x, nrm)
       use dqrm_spmat_mod
       real(kind(1.d0))             :: nrm(:)
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0))             :: b(:,:), x(:,:)
     end subroutine dqrm_residual_norm2d
     subroutine dqrm_residual_norm1d(qrm_mat, b, x, nrm)
       use dqrm_spmat_mod
       real(kind(1.d0)) :: nrm
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0)) :: b(:), x(:)
     end subroutine dqrm_residual_norm1d
  end interface qrm_residual_norm



  interface dqrm_residual_orth
     module procedure dqrm_residual_orth2dw, dqrm_residual_orth1dw
     module procedure dqrm_residual_and_orth2dw, dqrm_residual_and_orth1dw
  end interface dqrm_residual_orth

  interface qrm_residual_orth
     subroutine dqrm_residual_orth2d(qrm_mat, r, nrm)
       use dqrm_spmat_mod
       real(kind(1.d0))             :: nrm(:)
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0))             :: r(:,:)
     end subroutine dqrm_residual_orth2d
     subroutine dqrm_residual_orth1d(qrm_mat, r, nrm)
       use dqrm_spmat_mod
       real(kind(1.d0))             :: nrm
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0))             :: r(:)
     end subroutine dqrm_residual_orth1d
     subroutine dqrm_residual_and_orth2d(qrm_mat, b, x, nrm)
       use dqrm_spmat_mod
       real(kind(1.d0))             :: nrm(:)
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0))             :: b(:,:), x(:,:)
     end subroutine dqrm_residual_and_orth2d
     subroutine dqrm_residual_and_orth1d(qrm_mat, b, x, nrm)
       use dqrm_spmat_mod
       real(kind(1.d0))             :: nrm
       type(dqrm_spmat_type) :: qrm_mat
       real(kind(1.d0))             :: b(:), x(:)
     end subroutine dqrm_residual_and_orth1d
  end interface qrm_residual_orth


contains

  subroutine dqrm_least_squares2dw(qrm_mat, b, x)
    use dqrm_spmat_mod
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0)) :: b(:,:), x(:,:)
    call dqrm_least_squares2d(qrm_mat, b, x)
    return
  end subroutine dqrm_least_squares2dw

  subroutine dqrm_least_squares1dw(qrm_mat, b, x)
    use dqrm_spmat_mod
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0)) :: b(:), x(:)
    call dqrm_least_squares1d(qrm_mat, b, x)
    return
  end subroutine dqrm_least_squares1dw
  

  subroutine dqrm_min_norm2dw(qrm_mat, b, x)
    use dqrm_spmat_mod
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0)) :: b(:,:), x(:,:)
    call dqrm_min_norm2d(qrm_mat, b, x)
    return
  end subroutine dqrm_min_norm2dw

  subroutine dqrm_min_norm1dw(qrm_mat, b, x)
    use dqrm_spmat_mod
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0)) :: b(:), x(:)
    call  dqrm_min_norm1d(qrm_mat, b, x)
    return
  end subroutine dqrm_min_norm1dw


  subroutine dqrm_residual_norm2dw(qrm_mat, b, x, nrm)
    use dqrm_spmat_mod
    real(kind(1.d0))             :: nrm(:)
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0))             :: b(:,:), x(:,:)
    call dqrm_residual_norm2d(qrm_mat, b, x, nrm)
    return
  end subroutine dqrm_residual_norm2dw

  subroutine dqrm_residual_norm1dw(qrm_mat, b, x, nrm)
    use dqrm_spmat_mod
    real(kind(1.d0)) :: nrm
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0)) :: b(:), x(:)
    call dqrm_residual_norm1d(qrm_mat, b, x, nrm)
    return
  end subroutine dqrm_residual_norm1dw
  


  subroutine dqrm_residual_orth2dw(qrm_mat, r, nrm)
    use dqrm_spmat_mod
    real(kind(1.d0))             :: nrm(:)
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0))             :: r(:,:)
    call dqrm_residual_orth2d(qrm_mat, r, nrm)
    return
  end subroutine dqrm_residual_orth2dw

  subroutine dqrm_residual_orth1dw(qrm_mat, r, nrm)
    use dqrm_spmat_mod
    real(kind(1.d0))             :: nrm
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0))             :: r(:)
    call  dqrm_residual_orth1d(qrm_mat, r, nrm)
    return
  end subroutine dqrm_residual_orth1dw

  subroutine dqrm_residual_and_orth2dw(qrm_mat, b, x, nrm)
    use dqrm_spmat_mod
    real(kind(1.d0))             :: nrm(:)
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0))             :: b(:,:), x(:,:)
    call dqrm_residual_and_orth2d(qrm_mat, b, x, nrm)
    return
  end subroutine dqrm_residual_and_orth2dw

  subroutine dqrm_residual_and_orth1dw(qrm_mat, b, x, nrm)
    use dqrm_spmat_mod
    real(kind(1.d0))             :: nrm
    type(dqrm_spmat_type) :: qrm_mat
    real(kind(1.d0))             :: b(:), x(:)
    call dqrm_residual_and_orth1d(qrm_mat, b, x, nrm)
    return
  end subroutine dqrm_residual_and_orth1dw
  

end module dqrm_methods_mod
